//  Boost utility/nonderivable.hpp header file  ------------------//

//  (C) Copyright Manuel Fiorelli 2007. Distributed under the Boost
//  Software License, Version 1.0. (See accompanying file
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org/libs/utility for documentation.

#ifndef BOOST_UTILITY_NONDERIVABLE_HPP
#define BOOST_UTILITY_NONDERIVABLE_HPP

namespace boost
{

	/*
	 * Helper class with protected default constructor and
	 * destructor.
	 */
	class nonderivable_helper
	{
		protected:
			
			nonderivable_helper() {}
			
			/*
			 * A protected destructor ensures that objects
			 * of derived classes couldn't be destroyed through
			 * a base class pointer.
			 */
			~nonderivable_helper() {}
	};
	
}

/*
 * Usage ---------------------------------------------------------//
 *
 * Use BOOST_NON_DERIVABLE to define a class whose derived classes
 * shouldn't be instantiated.
 *
 * Suppose you want to define a class Foo, then you have to use the
 * macro as follows:
 *
 *    class Foo : BOOST_NON_DERIVABLE
 *    {
 *        ...
 *    };
 *
 * The class Foo is, in fact, pseudo-final, meaning that you are
 * free to subclass it, altough its derived classes cannot be
 * instantiated.
 * 
 * Techniques ----------------------------------------------------//
 *
 * When you use the BOOST_NON_DERIVABLE macro as suggested, in fact
 * you use boost::nonderivable_helper as private virtual base class.
 * Protected members are accessible within the direct subclass, but
 * they aren't in indirect subclasses, since you use the private
 * inheritance. Beside, the virtual base class has to be initialized
 * in every subclass (not only in the direct subclass), thus when
 * you try to instantiate an indirect subclass, you will attempt to
 * use an unaccessible member (the default constructor of 
 * nonderivable_helper), which produces a compilation error.
 */
#define BOOST_NON_DERIVABLE private virtual ::boost::nonderivable_helper

#endif // BOOST_UTILITY_NONDERIVABLE_HPP


